@@ -1,4 +1,4 @@
-// $Id: main_thread.c,v 1.3 2005/11/08 21:29:54 khlut Exp $
+// $Id: main_thread.c,v 1.4 2006/02/01 22:47:51 khlut Exp $
 // Copyright 2005 Elphel, Inc.
 //
 // This file is part of GenReS.
@@ -105,14 +105,16 @@ void send_value(PD *pd, char *key, char *value) //send variable to slave
 {
   PR_Lock(pd->lock);
   if(( (!pd->slavectrl) || fprintf(pd->slavectrl,"%s=%s\n",key,value)<=0 ) && pd->status==Running // try to reanimate slave
-     && ( strcmp(key,"run") || strcmp(value,"0") )) { 	// ignore run=0
+     && ( strcmp(key,"run") || strcmp(value,"0") )) { 	// ignore run=0 when slave is nonfunctional
     perror("write to pipe");
+   
+    PR_Unlock(pd->lock);
     wait_slave(pd);
     pd->status=Init;
     run_slave(pd,pd->mime);	// run slave parallel process
     
     // slave observer thread
-    if(!( pd->observer=PR_CreateThread( PR_USER_THREAD, (void (*)(void*))&observer_thread,
+    if(!( pd->observer=PR_CreateThread(PR_USER_THREAD, (void (*)(void*))&observer_thread,
                                   pd, PR_PRIORITY_HIGH,
                                   PR_GLOBAL_THREAD, PR_JOINABLE_THREAD, 0) )) return;
     // send all current variables
@@ -126,6 +128,16 @@ void send_value(PD *pd, char *key, char *value) //send variable to slave
     pd->xid && fprintf(pd->slavectrl,"xid=%x\n",(unsigned)pd->xid);
     fprintf(pd->slavectrl,"%s=%s\n",key,value);
     pd->status=Running;
+    
+    return;
+    
+  }else if(pd->status==Running && !strcmp(key,"run") && !strcmp(value,"0") && pd->slavectrl) {
+    PR_Unlock(pd->lock);
+    wait_slave(pd);
+    pd->status=Running;
+
+    return;
+      
   }
   PR_Unlock(pd->lock);  
 }
@@ -251,11 +263,11 @@ NPError NP_GetValue(void *future, NPPVariable variable, void *value)
   printf("NP_GETVALUE\n");
   switch (variable) {
     case NPPVpluginNameString:
-      *((char **) value) = "The GenReS Mozilla plugin V 0.7.1";
+      *((char **) value) = "The GenReS Mozilla plugin V 0.8.0";
       break;
     case NPPVpluginDescriptionString:
       *((char **) value) = "<a href=https://savannah.nongnu.org/projects/genres>GENeric REconfigureable Scriptable plugin</a> for Mozilla/Firefox<br>"
-    			   "Version 0.7.1<br>"
+    			   "Version 0.8.0<br>"
 			   "Author: Sergey Khlutchin<br>"
 			   LICENSE;
       break;
@@ -289,16 +301,16 @@ NPError NP_Initialize(NPNetscapeFuncs * moz_funcs, NPPluginFuncs * plugin_funcs)
   plugin_funcs->getvalue = NPP_GetValue;
   plugin_funcs->setwindow = NPP_SetWindow;
   plugin_funcs->destroy = NPP_Destroy;
-//	functions undefined
-//  plugin_funcs->setvalue = NPP_SetValue;
   plugin_funcs->newstream = NPP_NewStream;
+//	functions undefined
 //  plugin_funcs->destroystream = NPP_DestroyStream;
-//  plugin_funcs->asfile = NPP_StreamAsFile;
+//  plugin_funcs->setvalue = NPP_SetValue;
 //  plugin_funcs->writeready = NPP_WriteReady;
 //  plugin_funcs->write = NPP_Write;
+//  plugin_funcs->asfile = NPP_StreamAsFile;
 //  plugin_funcs->print = NPP_Print;
-//  plugin_funcs->event = NPP_HandleEvent;
 //  plugin_funcs->urlnotify = NPP_URLNotify;
+//  plugin_funcs->event = NPP_HandleEvent;
 
   return NPERR_NO_ERROR;
 }
@@ -371,7 +383,7 @@ void free_pd(PD *pd) { // free plugin data if reference counter zeroes
     instance_cnt--;
 }
 
-void wait_slave(PD * pd) {
+void wait_slave(PD * pd) { // wait for slave termination
     NPVariant *var;	int i;
     do{
         PR_Sleep(100);
@@ -385,6 +397,7 @@ void wait_slave(PD * pd) {
     }while(var->value.stringValue.utf8characters[0]=='1');
 
     printf("pdata refcnt=%d\n", pd->refcnt);
+    
     pd->status=Cancelling;
     PR_Sleep(100);
     process_pending_events(pd);
@@ -436,7 +449,7 @@ NPError NPP_GetValue(NPP instance, NPPVariable variable, void *value)
 }
 
 
-NPError NPP_SetWindow (NPP instance, NPWindow* npwindow)
+NPError NPP_SetWindow(NPP instance, NPWindow* npwindow)
 {
     Window xid= (Window)npwindow->window;
     
@@ -472,14 +485,14 @@ typedef struct {
 void * PR_CALLBACK HandleEvent(ObserveEvent* event);
 void PR_CALLBACK DestroyEvent(ObserveEvent* event);
 
-PLEvent * strevent( PD *pd, const char * str)  // make event from a string
+PLEvent * strevent( PD *pd, const char * str)  // make event from a string (is called from other thread)
 {
     if (!pd) return 0;
     PR_Lock(pd->lock);
     if(!pd->instance)	{ PR_Unlock(pd->lock); return 0; }
     ObserveEvent *event = PR_MALLOC(sizeof(ObserveEvent));
     if (!event)		{ PR_Unlock(pd->lock); return 0; }
-    printf("newEvent %d %s\n", (int)event, str);
+    printf("newEvent %u %s\n", (int)event, str);
     
     PL_InitEvent( &event->e, pd, HandleEvent, DestroyEvent);
     event->str=PR_Malloc(strlen(str)+3);
@@ -495,7 +508,7 @@ void PR_CALLBACK DestroyEvent(ObserveEvent* event)
 {
     if(event) {
 	PR_Lock(event->pd->lock);
-	printf("DestroyEvent %d ", (int)event);
+	printf("DestroyEvent %u ", (int)event);
 	printf("pdata refcnt=%d\n", event->pd->refcnt);
 	free_pd(event->pd);
 	if(event->str) PR_Free(event->str);
@@ -538,7 +551,7 @@ void * PR_CALLBACK HandleEvent(ObserveEvent* event)
 		)
 	    ) {
 		printf("evaluate ");
-		//In "evaluate" instance can be desroyed
+		//In "evaluate" the instance can be desroyed
 		//NPN.evaluate(event->pd->instance, obj, &(val.value.stringValue), 0);
 		char *url=PR_Malloc(val.value.stringValue.utf8length+12);
 		strcpy(url,"javascript:");
